#!/usr/bin/env python
# -- coding: utf-8

#       Copyright 2007 Manuel Vazquez Acosta <mva.led@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

"""
Introduction
------------

Guglebot is the spider created
for Gugle project.

Guglebot's main job is to update/increase Gugle's
knowledge base.

Current Goals: Fetch (X)HTML and XML pages.

Guglebot uses standard techniques to do its job:
    1.  Try to fetch each of its known pages (which
        have became dirty) if changed.
    1.1 In that case, it then extracts links
        from the downloaded page and add them
        if necesary to its to-fetch list.
    2.  For each downloaded page, either brand
        new or updated, it passes it to the indexer
        Asociate meta data, as its downloaded date.
        And establish a come-dirty timestamp so
        Guglebot knows when to download again that
        page.

Guglebot WILL HAVE other techniques to add new pages:
    1. LDAP search, for instance
    2. Port scan."""


from gugle.bot.workerbot import WorkerBot
from gugle.storage.kb import BasicKnowledgeBase, BasicKBEntry

import threading

USERAGENT_STRING_UNICODE = u'Guglebot (Alpha/$Rev: 478 $)'
USERAGENT_STRING = 'Guglebot (Alpha/$Rev: 478 $)'

MAX_AMOUNT_OF_WORKERS = 1

class Guglebot:
    """Guglebot class"""

    def __init__(self, BKB = None):
        if BKB == None:
            self.__BKB_ = BasicKnowledgeBase()
        else:
            self.__BKB_ = BKB

        self.__workers_ = []
        self.__killing = False

    def put(self, url):
        item = BasicKBEntry(url)
        self.__BKB_.put(item)


    def run(self):
        """Creates and starts MAX_AMOUNT_OF_WORKERS
        WorkerBots, then waits till all of them are down.

        $Rev: 478 $
        If a KeyboardInterrupt is detected we stop'em all.
        """

        try:
            for i in range(0, MAX_AMOUNT_OF_WORKERS):
                self.__workers_.append(WorkerBot(self.__BKB_, autoStart=True))

            while self.alive():
                threading._sleep(10)
        except KeyboardInterrupt:
            #print "Guglebot: KeyboardInterrupt detected! Stopping"
            #print "DON'T PRESS CTRL-C AGAIN"

            self.kill()
            self._joinWorkers()

            # So we can detect Ctrl+C in main
            raise

        self.__BKB_.dump()

    def alive(self):
        i = 0
        while i < MAX_AMOUNT_OF_WORKERS and self.__workers_[i].isAlive():
            i = i + 1

        return i != 0

    def putMemoryLimit(self, limit):
        self.__BKB_.__limit__ = limit

    def addURLFilter(self, url, policy):
        self.__BKB_.addURLFilter(url, policy)

    def report(self):
        self.__BKB_.dump()

    def kill(self):
        print "Guglebot is about to stop!!!!"

        self.__killing = True
        for i in range(0, MAX_AMOUNT_OF_WORKERS):
            self.__workers_[i].Terminate()

    def _joinWorkers(self):
        for i in range(0, MAX_AMOUNT_OF_WORKERS):
            if self.__workers_[i].isAlive():
                self.__workers_[i].join()
