#!/usr/bin/env python
# -- coding: latin-1

#       Copyright 2007 Manuel Vazquez Acosta <mva.led@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import sys
import re

from gugle.storage.kb import BasicKnowledgeBase, GugleRevisionNumber
from copy import deepcopy

JRPattern = re.compile(r'^http://(www\.)?juventudrebelde\.cu/[^/]+/\d{4}-\d{2}-\d{2}/[-a-zA-Z]+/?$')
QRPattern = re.compile(r'^http://www\.(vanguardia\.co|cubadebate)\.cu/.*?newsid(?:_obj_id)?=(\d+).*$')

def ConvertToCanonical(url):
    Match = QRPattern.match(url)
    if Match:
        return 'http://www.%s.cu/_x_canonical/newsid=%s' % (Match.group(1), Match.group(2))
    else:
        return url

def main():
    KB = BasicKnowledgeBase()

    FilteredKB = BasicKnowledgeBase(fs='var/filtered-%s.db' % GugleRevisionNumber)

    FilteredKB.addURLFilter(JRPattern, "allow")
    FilteredKB.addURLFilter(QRPattern, "allow")


    for i in range(0, len(KB.__BKB__)):
        Entry = deepcopy(KB.__BKB__.get(i))
        Entry.url = ConvertToCanonical(Entry.url)
        try:
            FilteredKB.put(Entry)
        except:
            print "Exception thrown putting entry #%s" % i

    for (referrer, target) in KB.__WebGraph__.generateGugleMappings():
        referrer = ConvertToCanonical(referrer)
        target = ConvertToCanonical(target)

        if FilteredKB.__BKB__.__Entries__.has_key(referrer) and FilteredKB.__BKB__.__Entries__.has_key(target):
            FilteredKB.__WebGraph__.registerGugleLink(referrer, target)

    print "BKB len: %d" % len(FilteredKB.__BKB__)
    print "Blacklist len: %d" % len(FilteredKB.__BlackList__)
    print "WebGraph len: %d" % len(FilteredKB.__WebGraph__)

if __name__ == '__main__':
    main()
