#!/usr/bin/env python
# -- coding: utf-8

#       Copyright 2007 Manuel Vazquez Acosta <mva.led@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

"""WorkerBot

$Id: workerbot.py 478 2007-12-10 16:21:38Z manu $ """

from gugle.bot.linkbot import Linkbot
from gugle.storage.kb import BasicKnowledgeBase, BasicKBEntry, Empty

import threading
from threading import Thread

import datetime
import urlparse, urllib2

# This is the amount of seconds we shall wait before attempting
# to fetch another page. This is not a per-server parameter, but
# a global one.
NICENESS=2

class WorkerBot(Thread):
    """
    WorkerBot/$Rev: 478 $

    A single worker thread that (1) fetches a single page
    which URL is dequeued from a Queue, (2) screen-scraps the
    links in the page, (3) puts'em back into the Queue.

    If the Queue has no entries the thread blocks until notified.
    A Condition/Event is used in this case.

    The WorkBot is a committed worker that will refuse to stop doing
    its job. The ONLY way for it to stop working is calling Terminate(),
    otherwise it'll wait until a task is assigned for it.

    params:
    URLQueue: As for this revision ($Rev: 478 $), just a (synchronized) Queue
    of URLs. Later it should be a guglecommon.kb.BasicKnowledgeBase

"""

    def __init__(self, BasicKB, autoStart = True):
        assert BasicKB != None and isinstance(BasicKB, BasicKnowledgeBase)

        Thread.__init__(self)

        self.Queue = BasicKB
        self.__terminate = False;

        if autoStart:
            self.start()

    def Terminate(self):
        print "Workerbot %s: Somebody wants to stop me" % self.getName()
        self.__terminate = True

    def FetchURL(self, url):
        #We refuse to fetch non-httpish URIs
        if self._canFetchURL(url):
            print "Workerbot %s: Fetching %s" % (self.getName(), url)
            try:
                #TODO: Consider using httplib instead. urrlib2 default timeout
                #is too long.
                fh = urllib2.urlopen(url)
                html = fh.read()

                linkbot = Linkbot(url, html)
                links = linkbot.getLinks()
                for which in links:
                    if self._canFetchURL(which) and not self.Queue.has_key(which):
                        print "Workerbot %s: Adding %s to BKB" % (self.getName(), which)
                        self.Queue.put(BasicKBEntry(which), referrer = url)
                return True
            except urllib2.HTTPError, http_error:
                self.Queue.put(BasicKBEntry(url, httpResponseCode=http_error.code))
            except urllib2.URLError:
                print "Workerbot %s: URLError while fetching %s" % (self.getName(), url)
                self.Queue.registerFailure(url)
                return False
            except:
                raise

    def _canFetchURL(self, url):
        scheme = urlparse.urlparse(url)[0]
        return scheme == 'http' or scheme == 'https'

    def run(self):
        while not self.__terminate:
            try:
                which = self.Queue.get(timeout=5)

                if not self.FetchURL(which.url):
                    print "Workerbot %s: Cannot fetch %s" % (self.getName(), which.url)
                else:
                    which.timestamp = datetime.datetime.now()

                # Be nice with web servers
                threading._sleep(NICENESS)
            except Empty:
                # Give a chance to the queue to be fed
                threading._sleep(3*NICENESS)

        print "Workbot %s: Finished" % self.getName()
